﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

namespace HPLC
{
    public unsafe class DevHPLC
    {
        #region Интерфейс с libusb

        #region Константы

        public const int LIBUSB_PATH_MAX = 512;
        const string LIBUSB_NATIVE_LIBRARY = "libusb0.dll";

        // Тип запроса (bmRequestType) формируется побитовым "или" 
        // констант из трех групп ниже.
        const byte USB_ENDPOINT_IN = 0x80;
        const byte USB_ENDPOINT_OUT = 0x00;

        const byte USB_TYPE_STANDARD = (0x00 << 5);
        const byte USB_TYPE_CLASS = (0x01 << 5);
        const byte USB_TYPE_VENDOR = (0x02 << 5);
        const byte USB_TYPE_RESERVED = (0x03 << 5);

        const byte USB_RECIP_DEVICE = 0x00;
        const byte USB_RECIP_INTERFACE = 0x01;
        const byte USB_RECIP_ENDPOINT = 0x02;
        const byte USB_RECIP_OTHER = 0x03;

        // Коды стандартных запросов (bRequest) с номерами 0 - 12 зарезервированы 
        // в спецификации USB поэтому все пользовательские будут начинаться с RQ_USER_BEG
        const byte RQ_USER_BEG = 13;

        #endregion

        #region Структуры

        /* Оригинальная структура из usb.h
        struct usb_endpoint_descriptor {
            unsigned char  bLength;
            unsigned char  bDescriptorType;
            unsigned char  bEndpointAddress;
            unsigned char  bmAttributes;
            unsigned short wMaxPacketSize;
            unsigned char  bInterval;
            unsigned char  bRefresh;
            unsigned char  bSynchAddress;

            unsigned char *extra;	// Extra descriptors
            int extralen;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_endpoint_descriptor
        {
            public byte bLength;
            public byte bDescriptorType;
            public byte bEndpointAddress;
            public byte bmAttributes;
            public byte wMaxPacketSize;
            public byte bInterval;
            public byte bRefresh;
            public byte bSynchAddress;
            public byte* extra;	// Extra descriptors
            public int extralen;
        };

        /* Оригинальная структура  из usb.h
        struct usb_interface_descriptor {
            unsigned char  bLength;
            unsigned char  bDescriptorType;
            unsigned char  bInterfaceNumber;
            unsigned char  bAlternateSetting;
            unsigned char  bNumEndpoints;
            unsigned char  bInterfaceClass;
            unsigned char  bInterfaceSubClass;
            unsigned char  bInterfaceProtocol;
            unsigned char  iInterface;
            struct usb_endpoint_descriptor *endpoint;
            unsigned char *extra;	// Extra descriptors 
            int extralen;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_interface_descriptor
        {
            public byte bLength;
            public byte bDescriptorType;
            public byte bInterfaceNumber;
            public byte bAlternateSetting;
            public byte bNumEndpoints;
            public byte bInterfaceClass;
            public byte bInterfaceSubClass;
            public byte bInterfaceProtocol;
            public byte iInterface;
            public usb_endpoint_descriptor* endpoint;
            public byte* extra;	// Extra descriptors 
            public int extralen;
        };

        /* Оригинальная структура  из usb.h
        struct usb_interface {
            struct usb_interface_descriptor *altsetting;
            int num_altsetting;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_interface
        {
            public usb_interface_descriptor* altsetting;
            public int num_altsetting;
        };

        /* Оригинальная структура  из usb.h
        struct usb_config_descriptor {
            unsigned char  bLength;
            unsigned char  bDescriptorType;
            unsigned short wTotalLength;
            unsigned char  bNumInterfaces;
            unsigned char  bConfigurationValue;
            unsigned char  iConfiguration;
            unsigned char  bmAttributes;
            unsigned char  MaxPower;
            struct usb_interface *interface;
            unsigned char *extra;	// Extra descriptors
            int extralen;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_config_descriptor
        {
            public byte bLength;
            public byte bDescriptorType;
            public ushort wTotalLength;
            public byte bNumInterfaces;
            public byte bConfigurationValue;
            public byte iConfiguration;
            public byte bmAttributes;
            public byte MaxPower;
            public usb_interface* pinterface;
            public byte* extra;	// Extra descriptors
            public int extralen;
        };

        /*  Оригинальная структура  из usb.h
        struct usb_device_descriptor {
            unsigned char  bLength;
            unsigned char  bDescriptorType;
            unsigned short bcdUSB;
            unsigned char  bDeviceClass;
            unsigned char  bDeviceSubClass;
            unsigned char  bDeviceProtocol;
            unsigned char  bMaxPacketSize0;
            unsigned short idVendor;
            unsigned short idProduct;
            unsigned short bcdDevice;
            unsigned char  iManufacturer;
            unsigned char  iProduct;
            unsigned char  iSerialNumber;
            unsigned char  bNumConfigurations;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_device_descriptor
        {
            public byte bLength;
            public byte bDescriptorType;
            public ushort bcdUSB;
            public byte bDeviceClass;
            public byte bDeviceSubClass;
            public byte bDeviceProtocol;
            public byte bMaxPacketSize0;
            public ushort idVendor;
            public ushort idProduct;
            public ushort bcdDevice;
            public byte iManufacturer;
            public byte iProduct;
            public byte iSerialNumber;
            public byte bNumConfigurations;
        };

        /*  Оригинальная структура  из usb.h
        struct usb_device 
        {
            struct usb_device *next, *prev;
            char filename[LIBUSB_PATH_MAX];
            struct usb_bus *bus;
            struct usb_device_descriptor descriptor;
            struct usb_config_descriptor *config;
            void *dev;		// Darwin support
            unsigned char devnum;
            unsigned char num_children;
            struct usb_device **children;
        }; */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_device
        {
            public usb_device* next;
            public usb_device* prev;
            public fixed byte filename[LIBUSB_PATH_MAX];
            public usb_bus* bus;
            public usb_device_descriptor descriptor;
            public usb_config_descriptor* config;
            public IntPtr dev;		// Darwin support
            public byte devnum;
            public byte num_children;
            public IntPtr children;
        };

        /* Оригинальная структура  из usb.h
        struct usb_bus 
        {
            struct usb_bus *next, *prev;
            char dirname[LIBUSB_PATH_MAX];
            struct usb_device *devices;
            unsigned long location;
            struct usb_device *root_dev;
        };
        */
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct usb_bus
        {
            public usb_bus* next;
            public usb_bus* prev;
            public fixed byte filename[LIBUSB_PATH_MAX];
            public usb_device* devices;
            public uint location;
            public usb_device* root_dev;
        };

        /* Оригинальная структура  из usbi.h
        struct usb_dev_handle {
            int fd;
            struct usb_bus *bus;
            struct usb_device *device;
            int config;
            int interface;
            int altsetting; 
            void *impl_info; // Added by RMT so implementations can store other per-open-device data
        };*/
        public struct usb_dev_handle
        {
            public int fd;
            public usb_bus* bus;
            public usb_device* device;
            public int config;
            public int iinterface;
            public int altsetting;
            public IntPtr impl_info; // Added by RMT so implementations can store other per-open-device data
        };
        #endregion

        #region Импорт функций libusb
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern void usb_init();
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern int usb_find_busses();
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern int usb_find_devices();
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern usb_bus* usb_get_busses();
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern usb_dev_handle* usb_open(usb_device* dev);
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern int usb_close(usb_dev_handle* dev);
        [DllImport(LIBUSB_NATIVE_LIBRARY)]
        public static extern int usb_control_msg(usb_dev_handle* dev, int requesttype, int request,
                                                 int value, int index, byte[] bytes, int size, int timeout);
        #endregion

        #endregion

        #region Адреса регистров и пины микроконтроллера
        public const byte aPINA = 0x19;
        public const byte aDDRA = 0x1A;
        public const byte aPORTA = 0x1B;

        public const byte aPINB = 0x16;
        public const byte aDDRB = 0x17;
        public const byte aPORTB = 0x18;

        public const byte aPINC = 0x13;
        public const byte aDDRC = 0x14;
        public const byte aPORTC = 0x15;

        public const byte aPIND = 0x10;
        public const byte aDDRD = 0x11;
        public const byte aPORTD = 0x12;

        public const byte PIN0 = 0;
        public const byte PIN1 = 1;
        public const byte PIN2 = 2;
        public const byte PIN3 = 3;
        public const byte PIN4 = 4;
        public const byte PIN5 = 5;
        public const byte PIN6 = 6;
        public const byte PIN7 = 7;
        #endregion

        #region Константы
        const int RQ_IO_READ        = 0x11;
        const int RQ_IO_WRITE       = 0x12;
        const int RQ_MEM_READ       = 0x13;
        const int RQ_MEM_WRITE      = 0x14;

        const int RQ_EOP_RES        = 0x1E;
        const int RQ_EOP_FLAG       = 0x1F;

        const int RQ_TEST           = 0x20;
        const int RQ_OW_WBIT        = 0x21;
        const int RQ_OW_WBYTE       = 0x22;
        const int RQ_OW_RBIT        = 0x23;
        const int RQ_OW_RBYTE       = 0x24;
        const int RQ_OW_RESET       = 0x25;

        const int RQ_OW_COMMAND     = 0x26;
        const int RQ_OW_FIRST       = 0x27;
        const int RQ_OW_NEXT        = 0x28;
        const int RQ_OW_GET_ROM     = 0x29;
        const int RQ_OW_SET_ROM     = 0x2A;
        const int RQ_OW_SEARCH_ALL  = 0x2B;
        const int RQ_OW_GET_ROMI    = 0x2C;

        const int RQ_DATA_READY     = 0x31;
        const int RQ_DATA_READ      = 0x32;
        const int RQ_END_FLAG_READ  = 0x33;

        #endregion

        #region Приватные члены
        private ushort vid, pid;
        private usb_dev_handle* handle = null;
        private usb_bus* bus;
        private usb_device* dev;
        private byte[] buffer = new byte[4];
        #endregion

        public DevHPLC(ushort vid, ushort pid)
        {
            this.vid = vid; this.pid = pid;
            usb_init();
            int fb = usb_find_busses();
            int fd = usb_find_devices();
            byte[] buffer = new byte[4];

            for (bus = usb_get_busses(); bus != null && handle == null; bus = bus->next)
                for (dev = bus->devices; dev != null; dev = dev->next)
                    if (dev->descriptor.idVendor == vid && dev->descriptor.idProduct == pid)
                    {
                        handle = usb_open(dev);
                        break;
                    }
        }
        public bool IsOpen()
        {
            return handle != null;
        }
        public bool Close()
        {
            if (handle == null)
                return false;
            usb_close(handle);
            return true;
        }

        #region Работа с устройством
        public byte dataReady   // Флаг готовности данных
        {
            get
            {
                usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
                                RQ_DATA_READY, 0, 0, buffer, 1, 5000);

                return buffer[0];
            }
        }
        public byte[] data      // Актуальные данные с детектора 
        {
            get
            {
                usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
                            RQ_DATA_READ, 0, 0, buffer, 3, 5000);
                return buffer;
            }
        }
        public bool endFlag     // Флаг конца цикла детектора
        {
            get
            {
                usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
                                RQ_END_FLAG_READ, 0, 0, buffer, 1, 5000);

                return (buffer[0] == 0x01);
            }
        }

        #endregion
    }
}
