﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using ZedGraph;

namespace HPLC
{
    public partial class FormMain : Form
    {
        ushort vid = 0x16C0, pid = 0x05DC;
        DevHPLC dev;
        byte[] buf = new byte[3];
        Color c, gc = Color.LightGray;
        double[] t, a;  // Точки хроматограммы
        int ci;         // Счетчик точек хроматограммы
        double st, ct;  // Поправка для учета пауз и текущее время хроматограммы (без учета пауз)
        DateTime t0;
        GraphPane pane;
        PointPairList aList;
        LineItem aCurve;
        bool runFlag = false, modFlag = false;
        FormSpectrum fs;

        Random rnd = new Random();

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            dev = new DevHPLC(vid, pid);
            if (!dev.IsOpen())
            {
                MessageBox.Show(String.Format("Невозможно найти устройство vid = 0x{0:X}, pid = 0x{1:X}", vid, pid),
                                              "Ошибка USB", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();

            }
            else
            {
                t = new double[10000];
                a = new double[10000];
                ci = 0; ct = 0; st = 0;
                c = toolStripStatusLabel.ForeColor;

                #region Инициализация zedGraph
                pane = zedGraphControl.GraphPane;
                pane.Title.IsVisible = false;
                pane.Fill = new Fill(Color.Black);
                pane.Chart.Fill = new Fill(Color.Black);
                pane.Chart.Border.Color = gc;

                pane.XAxis.Title.IsVisible = false;
                pane.XAxis.Scale.MaxAuto = true;
                pane.XAxis.Scale.FontSpec.Size = 12;
                pane.XAxis.MajorGrid.IsVisible = true;
                pane.XAxis.Color = gc;
                pane.XAxis.Scale.FontSpec.FontColor = gc;
                pane.XAxis.MajorGrid.Color = gc;

                pane.YAxis.Title.IsVisible = false;
                pane.YAxis.Scale.Min = -5000;
                pane.YAxis.Scale.Max = 40000;
                pane.YAxis.Scale.FontSpec.Size = 12;
                pane.YAxis.MajorGrid.IsVisible = true;
                pane.YAxis.Color = gc;
                pane.YAxis.Scale.FontSpec.FontColor = gc;
                pane.YAxis.MajorGrid.Color = gc;

                pane.YAxis.ScaleFormatEvent += new Axis.ScaleFormatHandler(YScaleFormatEvent);

                aList = new PointPairList();
                aCurve = pane.AddCurve("", aList, Color.Orange, SymbolType.None);

                pane.YAxis.MajorGrid.IsZeroLine = false;
                zedGraphControl.AxisChange();
                #endregion
                
                fs = new FormSpectrum(this.dev);
                toolStripButtonSpectrum.Enabled = true;
            }
        }

        public string YScaleFormatEvent(GraphPane pane, Axis axis, double val, int index)
        {
            return val.ToString("f0");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int wl, wh;
            int w;
            bool sign;
            TimeSpan ts;

            if (ci >= 10000) // Запас большой, но не бесконечный - дошли до конца
            {
                toolStripButtonStartStop_Click(sender, e);
                MessageBox.Show("Дошли до конца...");
                return;
            }

            if (dev.dataReady != 1) // Данные еще не готовы...
                return;

            ts = DateTime.Now - t0;
            ct = ts.Minutes + (ts.Seconds + ts.Milliseconds * 0.001d) / 60.0 - st; // Время - в минутах
            t[ci] = ct;
            
            buf = dev.data; // Читаем буфер с данными
//            buf[0] = (byte)rnd.Next(255);
//            buf[1] = (byte)(64 + rnd.Next(32));
//            buf[2] = 0x01;

            wh = buf[1]; sign = (wh & 0x0080) != 0; wh &= 0x7F; // Выделим знак и сбросим этот бит
            wh <<= 8; wl = buf[0]; w = wh | wl; w = 32768 - w; // Формируем 15-разрядное число
            if ((buf[2] & 0x01) == 0) // Анализируем наличие переполнения - в данном случае оно есть
            {
                w += 32768; // Модифицируем данные
                toolStripStatusLabel.ForeColor = Color.Red; // Подсветим поле вывода в случае переполнения
            }
            else
                toolStripStatusLabel.ForeColor = c;
            if (sign) // Учтем знак
                w = -w;
            a[ci] = w;

            toolStripStatusLabel.Text = "[" + ci.ToString() + "] = [" + t[ci].ToString("f3") + " : " + a[ci].ToString("f0") + "]";

            aList.Add(t[ci], a[ci]); // Добавим точку на график
            zedGraphControl.AxisChange();
            zedGraphControl.Refresh();

            if (!modFlag) // Установим флаг модификации данных
            {
                modFlag = true;
                this.Text = "HPLC*";
            }
            ci++;
        }

        private void toolStripButtonStartStop_Click(object sender, EventArgs e)
        {
            TimeSpan ts;
            double tsm;

            if (!runFlag) // Запускаем запись хроматограммы
            {
                timer1.Enabled = true;
                runFlag = true;
                toolStripButtonStartStop.ToolTipText = "Остановить запись хроматограммы";
                toolStripButtonStartStop.Image = HPLC.Properties.Resources.Stop;
                toolStripButtonSpectrum.Enabled = false;
                if (ci == 0) // Начало новой хроматограммы
                {
                    t0 = DateTime.Now;
                    st = 0;
                }
                else        // Запуск после паузы
                {
                    ts = DateTime.Now - t0;
                    tsm = ts.Minutes + (ts.Seconds + ts.Milliseconds * 0.001d) / 60.0;
                    st = tsm - t[ci - 1];   // Корректируем смещение
                }
            }
            else  // Остановка записи хроматограммы
            {
                timer1.Enabled = false;
                runFlag = false;
                toolStripButtonStartStop.ToolTipText = "Включить запись хроматограммы";
                toolStripButtonStartStop.Image = HPLC.Properties.Resources.Play;
                toolStripButtonSpectrum.Enabled = true;
            }
        }

        private void toolStripButtonSave_Click(object sender, EventArgs e)
        {
            StreamWriter sw;
            string s = "";
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                for (int i = 0; i < ci; i++)
                    s += t[i].ToString("f5") + "\t" + a[i].ToString() + "\r\n";
                using (sw = new StreamWriter(saveFileDialog.FileName, false, Encoding.UTF8))
                {
                    sw.Write(s);
                    sw.Close();
                }
                modFlag = false;
                this.Text = "HPLC";
            }
        }

        private void toolStripButtonCopy_Click(object sender, EventArgs e)
        {
            string s = "";

            for (int i = 0; i < ci; i++)
                s += t[i].ToString() + "\t" + a[i].ToString() + "\r\n";
            Clipboard.SetText(s);
            MessageBox.Show("Данные скопированы в буфер обмена");
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (ci > 0)
                if (modFlag)
                    if (MessageBox.Show("Данные будут утеряны. Завершить приложение?", "Предупреждение", MessageBoxButtons.YesNo) == DialogResult.No)
                    {
                        e.Cancel = true;
                        return;
                    }
            e.Cancel = false;
        }

        private void toolStripButtonNew_Click(object sender, EventArgs e)
        {
            if (ci > 0)
                if (modFlag)
                    if (MessageBox.Show("Данные будут утеряны. Продолжить?", "Предупреждение", MessageBoxButtons.YesNo) == DialogResult.No)
                        return;
            ci = 0; ct = 0;
            modFlag = false;
            this.Text = "HPLC";
            aList.Clear();
            zedGraphControl.Refresh();
        }

        private void toolStripButtonSpectrum_Click(object sender, EventArgs e)
        {
            fs.ShowDialog();
        }
    }
}
