﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ZedGraph;
using System.IO;

namespace HPLC
{
    public partial class FormSpectrum : Form
    {
        DevHPLC dev;
        byte[] buf = new byte[3];
        Color c, gc = Color.LightGray;
        int[] lam, val;
        int ci;
        GraphPane pane;
        PointPairList pList, gList;
        LineItem pCurve, gCurve;
        bool runFlag = false;

        Random rnd = new Random();

        public FormSpectrum(DevHPLC dev)
        {
            InitializeComponent();
            this.dev = dev;

            lam = new int[86];
            val = new int[86];
            for (int i = 0; i < 86; i++)
                lam[i] = 190 + 2 * i;
            ci = 0;
            c = toolStripStatusLabel.ForeColor;

            #region Инициализация zedGraph
            pane = zedGraphControl.GraphPane;
            pane.Title.IsVisible = false;
            pane.Fill = new Fill(Color.Black);
            pane.Chart.Fill = new Fill(Color.Black);
            pane.Chart.Border.Color = gc;

            pane.XAxis.Title.IsVisible = false;
            pane.XAxis.Scale.Min = 190;
            pane.XAxis.Scale.Max = 360;
            pane.XAxis.Scale.FontSpec.Size = 12;
            pane.XAxis.MajorGrid.IsVisible = true;
            pane.XAxis.Color = gc;
            pane.XAxis.Scale.FontSpec.FontColor = gc;
            pane.XAxis.MajorGrid.Color = gc;

            pane.YAxis.Title.IsVisible = false;
            pane.YAxis.Scale.Min = -5000;
            pane.YAxis.Scale.Max = 40000;
            pane.YAxis.Scale.FontSpec.Size = 12;
            pane.YAxis.MajorGrid.IsVisible = true;
            pane.YAxis.Color = gc;
            pane.YAxis.Scale.FontSpec.FontColor = gc;
            pane.YAxis.MajorGrid.Color = gc;

            pane.YAxis.ScaleFormatEvent += new Axis.ScaleFormatHandler(YScaleFormatEvent);

            pList = new PointPairList();
            pCurve = pane.AddCurve("", pList, Color.Orange, SymbolType.None);
            pCurve.Line.StepType = StepType.ForwardStep;

            gList = new PointPairList();
            gCurve = pane.AddCurve("", gList, Color.LightGray, SymbolType.None);
            gCurve.Line.StepType = StepType.ForwardStep;

            pane.YAxis.MajorGrid.IsZeroLine = false;
            zedGraphControl.AxisChange();
            #endregion
        }

        public string YScaleFormatEvent(GraphPane pane, Axis axis, double val, int index)
        {
            return val.ToString("f0");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int wl, wh;
            int w;
            bool sign;

            if (dev.endFlag || (ci >= 86)) // Дошли до конца спектра - сбросим счетчик
//            if ((ci >= 86)) // Затычка
            {
                ci = 0;
                return;
            }

            if (ci == 0) // Начинаем новый цикл измерений
            {
                gList.Clear(); // Перепишем все данные в список gList
                for (int i = 0; i < 86; i++)
                    gList.Add(lam[i], val[i]);
                pList.Clear();
                zedGraphControl.Refresh();
            }

            if (dev.dataReady != 1)
                return;

            buf = dev.data; // Читаем буфер с данными
//            buf[0] = (byte)rnd.Next(255);
//            buf[1] = (byte)rnd.Next(128);
//            buf[2] = 0x01;

            wh = buf[1]; sign = (wh & 0x0080) != 0; wh &= 0x7F; // Выделим знак и сбросим этот бит
            wh <<= 8; wl = buf[0]; w = wh | wl; w = 32768 - w; // Формируем 15-разрядное число
            if ((buf[2] & 0x01) == 0) // Анализируем наличие переполнения - в данном случае оно есть
            {
                w += 32768; // Модифицируем данные
                toolStripStatusLabel.ForeColor = Color.Red; // Подсветим поле вывода в случае переполнения
            }
            else
                toolStripStatusLabel.ForeColor = c;
            if (sign) // Учтем знак
                w = -w;
            val[ci] = w;

            toolStripStatusLabel.Text = "[" + lam[ci].ToString() + " : " + w.ToString() + "]";

            pList.Add(lam[ci], val[ci]); // Добавим точку на график
            gList.RemoveAt(0); // Уберем соответствующую точку из графика предыдущего цикла
            zedGraphControl.AxisChange();
            zedGraphControl.Refresh();

            ci++;
        }

        private void toolStripButtonStartStop_Click(object sender, EventArgs e)
        {
            runFlag = !runFlag;
            if (runFlag) // Цикл измерений включен
            {
                timer1.Enabled = true;
                toolStripButtonStartStop.ToolTipText = "Остановить запись спектра";
                toolStripButtonStartStop.Image = HPLC.Properties.Resources.Stop;
            }
            else
            {
                timer1.Enabled = false;
                toolStripButtonStartStop.ToolTipText = "Включить запись спектра";
                toolStripButtonStartStop.Image = HPLC.Properties.Resources.Play;
            }
        }

        private void FormSpectrum_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (runFlag) // Цикл измерений включен - выключим
            {
                runFlag = false;
                timer1.Enabled = false;
                toolStripButtonStartStop.ToolTipText = "Включить запись спектра";
                toolStripButtonStartStop.Image = HPLC.Properties.Resources.Play;
            }
        }

        private void toolStripButtonCopy_Click(object sender, EventArgs e)
        {
            string s = "";

            for (int i = 0; i < 86; i++)
                s += lam[i].ToString() + "\t" + val[i].ToString() + "\r\n";
            Clipboard.SetText(s);
            MessageBox.Show("Спектр скопирован в буфер обмена");
        }

        private void toolStripButtonSaveSpectrum_Click(object sender, EventArgs e)
        {
            StreamWriter sw;
            string s = "";
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                for (int i = 0; i < 86; i++)
                    s += lam[i].ToString() + "\t" + val[i].ToString() + "\r\n";
                using (sw = new StreamWriter(saveFileDialog.FileName, false, Encoding.UTF8))
                {
                    sw.Write(s);
                    sw.Close();
                }
            }
        }
    }
}
